package com.famoco.scanenginetestapp.data;

import android.content.Context;
import android.content.SharedPreferences;

import java.util.Set;

public class PreferencesManager {
    public static final String TRIGGER_MODE_KEY = "trigger_mode";
    public static final String REREAD_DELAY_KEY = "reread_delay";
    public static final String ENABLE_ALL_SYMBOLOGIES_KEY = "enable_all_symbologies";
    public static final String ENABLE_SYMBOLOGIES_KEY = "enable_symbologies";
    public static final String SUPPORT_INVERSE_COLOR_DETECTION_KEY
            = "support_inverse_color_detection";
    public static final String INVERSE_COLOR_DETECTION_ONLY_KEY = "inverse_color_detection_only";
    public static final String ACTIVATE_ILLUMINATION_LEVEL_KEY = "activate_illumination_level";
    public static final String ILLUMINATION_LEVEL_KEY = "illumination_level";
    public static final String LOCK_CONFIGURATION_KEY = "lock_configuration";
    public static final String NO_DATA_DISPLAY_KEY = "no_data_display";
    public static final String DETECTION_TIMEOUT_KEY = "detection_timeout";
    private static final String TAG = PreferencesManager.class.getSimpleName();
    private static final String PREF_FILE_NAME = "ScanEngineTestPreferencesFile";
    private static PreferencesManager sInstance;

    private SharedPreferences mPreferences;

    private PreferencesManager() {
        // nothing to do
    }

    public static synchronized PreferencesManager getInstance() {
        if (sInstance == null) {
            sInstance = new PreferencesManager();
        }
        return sInstance;
    }

    public void init(Context context) {
        mPreferences = context.getSharedPreferences(PREF_FILE_NAME, Context.MODE_PRIVATE);
    }

    public String getTriggerMode() {
        return mPreferences.getString(TRIGGER_MODE_KEY, "0");
    }

    public void setTriggerMode(String triggerMode) {
        mPreferences.edit().putString(TRIGGER_MODE_KEY, triggerMode).apply();
    }

    public String getRereadDelay() {
        return mPreferences.getString(REREAD_DELAY_KEY, "0");
    }

    public void setRereadDelay(String rereadDelay) {
        mPreferences.edit().putString(REREAD_DELAY_KEY, rereadDelay).apply();
    }

    public boolean getEnableAllSymbologies() {
        return mPreferences.getBoolean(ENABLE_ALL_SYMBOLOGIES_KEY, true);
    }

    public void setEnableAllSymbologies(boolean enableAllSymbologies) {
        mPreferences.edit().putBoolean(ENABLE_ALL_SYMBOLOGIES_KEY, enableAllSymbologies).apply();
    }

    public Set<String> getEnableSymbologies() {
        return mPreferences.getStringSet(ENABLE_SYMBOLOGIES_KEY, null);
    }

    public void setEnableSymbologies(Set<String> symbologies) {
        mPreferences.edit().putStringSet(ENABLE_SYMBOLOGIES_KEY, symbologies).apply();
    }

    public boolean getSupportInverseColorDetection() {
        return mPreferences.getBoolean(SUPPORT_INVERSE_COLOR_DETECTION_KEY, true);
    }

    public void setSupportInverseColorDetection(boolean supportInverseColor) {
        mPreferences.edit().putBoolean(SUPPORT_INVERSE_COLOR_DETECTION_KEY, supportInverseColor)
                .apply();
    }

    public boolean getInverseColorDetectionOnly() {
        return mPreferences.getBoolean(INVERSE_COLOR_DETECTION_ONLY_KEY, false);
    }

    public void setInverseColorDetectionOnly(boolean inverseColorDetectionOnly) {
        mPreferences.edit().putBoolean(INVERSE_COLOR_DETECTION_ONLY_KEY, inverseColorDetectionOnly)
                .apply();
    }

    public boolean getActivateIlluminationLevel() {
        return mPreferences.getBoolean(ACTIVATE_ILLUMINATION_LEVEL_KEY, true);
    }

    public void setActivateIlluminationLevel(boolean activateIlluminationLevel) {
        mPreferences.edit().putBoolean(ACTIVATE_ILLUMINATION_LEVEL_KEY, activateIlluminationLevel)
                .apply();
    }

    public String getIlluminationLevel() {
        return mPreferences.getString(ILLUMINATION_LEVEL_KEY, "0");
    }

    public void setIlluminationLevel(String illuminationLevel) {
        mPreferences.edit().putString(ILLUMINATION_LEVEL_KEY, illuminationLevel).apply();
    }

    public boolean getLockConfiguration() {
        return mPreferences.getBoolean(LOCK_CONFIGURATION_KEY, true);
    }

    public void setLockConfiguration(boolean lockConfiguration) {
        mPreferences.edit().putBoolean(LOCK_CONFIGURATION_KEY, lockConfiguration).apply();
    }

    public boolean getNoDataDisplay() {
        return mPreferences.getBoolean(NO_DATA_DISPLAY_KEY, false);
    }

    public void setNoDataDisplay(boolean noDataDisplay) {
        mPreferences.edit().putBoolean(NO_DATA_DISPLAY_KEY, noDataDisplay).apply();
    }

    public int getDetectionTimeout() {
        return mPreferences.getInt(DETECTION_TIMEOUT_KEY, 10000);
    }

    public void setDetectionTimeout(int detectionTimeout) {
        mPreferences.edit().putInt(DETECTION_TIMEOUT_KEY, detectionTimeout).apply();
    }
}
