package com.famoco.scanenginetestapp.ui.general_settings;

import android.os.Bundle;
import android.view.View;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.ViewModelProviders;
import androidx.preference.EditTextPreference;
import androidx.preference.ListPreference;
import androidx.preference.MultiSelectListPreference;
import androidx.preference.PreferenceFragmentCompat;
import androidx.preference.SwitchPreference;

import com.famoco.scanenginetestapp.R;
import com.famoco.scanenginetestapp.data.PreferencesManager;

import java.util.Set;


public class GeneralSettingsFragment extends PreferenceFragmentCompat {
    private static final String TAG = GeneralSettingsFragment.class.getSimpleName();

    private GeneralSettingsViewModel mGeneralSettingsViewModel;

    @Override
    public void onCreatePreferences(Bundle savedInstanceState, String rootKey) {
        mGeneralSettingsViewModel = ViewModelProviders.of(this)
                .get(GeneralSettingsViewModel.class);
        setPreferencesFromResource(R.xml.general_preferences, rootKey);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        view.setBackgroundColor(getContext().getResources().getColor(R.color.fragmentBackground));
        ListPreference triggerModePreference = findPreference(PreferencesManager.TRIGGER_MODE_KEY);
        ListPreference rereadDelayPreference = findPreference(PreferencesManager.REREAD_DELAY_KEY);
        SwitchPreference enableAllSymbologiesPreference =
                findPreference(PreferencesManager.ENABLE_ALL_SYMBOLOGIES_KEY);
        MultiSelectListPreference enableSymbologiesPreference =
                findPreference(PreferencesManager.ENABLE_SYMBOLOGIES_KEY);
        SwitchPreference supportInverseColorDetectionPreference =
                findPreference(PreferencesManager.SUPPORT_INVERSE_COLOR_DETECTION_KEY);
        SwitchPreference inverseColorDetectionOnlyPreference =
                findPreference(PreferencesManager.INVERSE_COLOR_DETECTION_ONLY_KEY);
        SwitchPreference activateIlluminationLevelPreference =
                findPreference(PreferencesManager.ACTIVATE_ILLUMINATION_LEVEL_KEY);
        ListPreference illuminationLevelPreference =
                findPreference(PreferencesManager.ILLUMINATION_LEVEL_KEY);
        SwitchPreference lockConfigurationPreference =
                findPreference(PreferencesManager.LOCK_CONFIGURATION_KEY);
        SwitchPreference noDataDisplayPreference =
                findPreference(PreferencesManager.NO_DATA_DISPLAY_KEY);
        EditTextPreference detectionTimeoutPreference =
                findPreference(PreferencesManager.DETECTION_TIMEOUT_KEY);

        triggerModePreference.setOnPreferenceChangeListener((preference, newValue) -> {
            if (!(newValue instanceof String)) {
                return false;
            }
            String value = (String) newValue;
            triggerModePreference.setValueIndex(Integer.parseInt(value));
            triggerModePreference.setSummary(triggerModePreference.getEntry());
            mGeneralSettingsViewModel.setTriggerMode(value);

            if (Integer.parseInt(value) < 2) {
                rereadDelayPreference.setEntries(R.array.manual_trigger_mode_reread_delay_entries);
                rereadDelayPreference
                        .setEntryValues(R.array.manual_trigger_mode_reread_delay_values);
            } else {
                rereadDelayPreference
                        .setEntries(R.array.presentation_trigger_mode_reread_delay_entries);
                rereadDelayPreference
                        .setEntryValues(R.array.presentation_trigger_mode_reread_delay_values);
            }
            mGeneralSettingsViewModel.setRereadDelay("0");
            rereadDelayPreference.setValueIndex(0);
            rereadDelayPreference.setSummary(rereadDelayPreference.getEntry());
            return false;
        });
        triggerModePreference.setSummary(triggerModePreference.getEntry());

        rereadDelayPreference.setOnPreferenceChangeListener((preference, newValue) -> {
            if (!(newValue instanceof String)) {
                return false;
            }
            String value = (String) newValue;
            if (Integer.parseInt(triggerModePreference.getValue()) < 2) {
                rereadDelayPreference.setValueIndex(Integer.parseInt(value));
            } else {
                rereadDelayPreference.setValueIndex(Integer.parseInt(value) - 3);
            }
            rereadDelayPreference.setSummary(rereadDelayPreference.getEntry());
            mGeneralSettingsViewModel.setRereadDelay(triggerModePreference.getValue());
            return false;
        });
        if (Integer.parseInt(triggerModePreference.getValue()) < 2) {
            rereadDelayPreference.setEntries(R.array.manual_trigger_mode_reread_delay_entries);
            rereadDelayPreference.setEntryValues(R.array.manual_trigger_mode_reread_delay_values);
        } else {
            rereadDelayPreference.setEntries(R.array.presentation_trigger_mode_reread_delay_entries);
            rereadDelayPreference
                    .setEntryValues(R.array.presentation_trigger_mode_reread_delay_values);
        }
        rereadDelayPreference.setSummary(rereadDelayPreference.getEntry());

        enableAllSymbologiesPreference.setOnPreferenceChangeListener((preference, newValue) -> {
            if (!(newValue instanceof Boolean)) {
                return false;
            }
            Boolean value = (Boolean) newValue;
            enableAllSymbologiesPreference.setChecked(value);
            mGeneralSettingsViewModel.setEnableAllSymbologies(value);
            return false;
        });
        enableAllSymbologiesPreference
                .setChecked(mGeneralSettingsViewModel.getEnableAllSymbologies());

        enableSymbologiesPreference.setOnPreferenceChangeListener((preference, newValue) -> {
            enableSymbologiesPreference.setValues((Set<String>) newValue);
            mGeneralSettingsViewModel.setEnableSymbologies((Set<String>) newValue);
            return false;
        });

        supportInverseColorDetectionPreference.setOnPreferenceChangeListener((preference,
                                                                              newValue) -> {
            if (!(newValue instanceof Boolean)) {
                return false;
            }
            Boolean value = (Boolean) newValue;
            supportInverseColorDetectionPreference.setChecked(value);
            mGeneralSettingsViewModel.setSupportInverseColorDetection(value);
            return false;
        });
        supportInverseColorDetectionPreference
                .setChecked(mGeneralSettingsViewModel.getSupportInverseColorDetection());

        inverseColorDetectionOnlyPreference
                .setOnPreferenceChangeListener((preference, newValue) -> {
            if (!(newValue instanceof Boolean)) {
                return false;
            }
            Boolean value = (Boolean) newValue;
            inverseColorDetectionOnlyPreference.setChecked(value);
            mGeneralSettingsViewModel.setInverseColorDetectionOnly(value);
            return false;
        });
        inverseColorDetectionOnlyPreference.setChecked(mGeneralSettingsViewModel
                .getInverseColorDetectionOnly());

        activateIlluminationLevelPreference
                .setOnPreferenceChangeListener((preference, newValue) -> {
            if (!(newValue instanceof Boolean)) {
                return false;
            }
            Boolean value = (Boolean) newValue;
            activateIlluminationLevelPreference.setChecked(value);
            mGeneralSettingsViewModel.setActivateIlluminationLevel(value);
            return false;
        });
        activateIlluminationLevelPreference.setChecked(mGeneralSettingsViewModel
                .getActivateIlluminationLevel());

        illuminationLevelPreference.setOnPreferenceChangeListener((preference, newValue) -> {
            if (!(newValue instanceof String)) {
                return false;
            }
            String value = (String) newValue;
            illuminationLevelPreference.setValueIndex(Integer.parseInt(value));
            illuminationLevelPreference.setSummary(illuminationLevelPreference.getEntry());
            mGeneralSettingsViewModel.setIlluminationLevel(value);
            return false;
        });
        illuminationLevelPreference.setSummary(illuminationLevelPreference.getEntry());

        lockConfigurationPreference.setOnPreferenceChangeListener((preference, newValue) -> {
            if (!(newValue instanceof Boolean)) {
                return false;
            }
            Boolean value = (Boolean) newValue;
            lockConfigurationPreference.setChecked(value);
            mGeneralSettingsViewModel.setLockConfiguration(value);
            return false;
        });
        lockConfigurationPreference.setChecked(mGeneralSettingsViewModel.getLockConfiguration());

        noDataDisplayPreference.setOnPreferenceChangeListener((preference, newValue) -> {
            if (!(newValue instanceof Boolean)) {
                return false;
            }
            Boolean value = (Boolean) newValue;
            noDataDisplayPreference.setChecked(value);
            mGeneralSettingsViewModel.setNoDataDisplay(value);
            return false;
        });
        noDataDisplayPreference.setChecked(mGeneralSettingsViewModel.getNoDataDisplay());

        detectionTimeoutPreference.setOnPreferenceChangeListener((preference, newValue) -> {
            if (!(newValue instanceof String)) {
                return false;
            }
            Integer value = Integer.parseInt((String) newValue);
            if (value < 0 || value > 30000) {
                Toast.makeText(getContext(), "value must be between 0 and 30000",
                        Toast.LENGTH_SHORT).show();
                return false;
            }
            detectionTimeoutPreference.setText("" + value);
            detectionTimeoutPreference.setSummary("" + value);
            mGeneralSettingsViewModel.setDetectionTimeout(value);
            return false;
        });
        detectionTimeoutPreference.setSummary("" + mGeneralSettingsViewModel.getDetectionTimeout());
    }
}
