package com.famoco.scanenginetestapp.ui.main_menu;

import android.media.AudioManager;
import android.media.ToneGenerator;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProviders;

import com.famoco.scanenginetestapp.R;

public class MainMenuFragment extends Fragment {

    private static final int TONE_VOLUME = 100; // from 0 to 100
    private static final int TONE_DURATION = 150; // in ms
    private MainMenuViewModel mMainMenuViewModel;
    private TextView mTextViewData;
    private TextView mTextViewFwVersion;
    private Button mDetectionButton;
    private ToneGenerator mToneGenerator =
            new ToneGenerator(AudioManager.STREAM_MUSIC, TONE_VOLUME);

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
        mMainMenuViewModel = ViewModelProviders.of(this).get(MainMenuViewModel.class);

        View root = inflater.inflate(R.layout.fragment_main_menu, container, false);
        mTextViewFwVersion = root.findViewById(R.id.firmware_text_view);
        mTextViewData = root.findViewById(R.id.data_text_view);
        mDetectionButton = root.findViewById(R.id.detection_button);

        mMainMenuViewModel.firmwareVersion.observe(getViewLifecycleOwner(), firmwareVersion -> {
            mTextViewFwVersion.setText(getResources().getString(R.string.fw_version,
                    firmwareVersion));
        });

        mMainMenuViewModel.barcodeData.observe(getViewLifecycleOwner(), barcodeData -> {
            mToneGenerator.startTone(ToneGenerator.TONE_CDMA_PIP, TONE_DURATION);
            mTextViewData.setText(barcodeData);
        });

        mMainMenuViewModel.scanningStateManual.observe(getViewLifecycleOwner(),
                scanningSateManual -> {
            switch (scanningSateManual) {
                case NONE:
                    mDetectionButton.setVisibility(View.GONE);
                    break;
                case ATTACHED:
                    // nothing to do
                    break;
                case READY:
                    mDetectionButton.setVisibility(View.VISIBLE);
                    mDetectionButton.setText(getString(R.string.start_barcode_scan));
                    mDetectionButton.setOnClickListener(view -> {
                        mMainMenuViewModel.startBarcodeDetection();
                    });
                    break;
                case SCANNING:
                    mDetectionButton.setVisibility(View.VISIBLE);
                    mDetectionButton.setText(getString(R.string.stop_barcode_scan));
                    mDetectionButton.setOnClickListener(view -> {
                        mMainMenuViewModel.stopBarcodeDetection();
                    });
                    break;
            }
        });

        return root;
    }

    @Override
    public void onResume() {
        super.onResume();
        mMainMenuViewModel.configure();
    }
}
